#include "llADT.h"

using namespace std;

// Constructor
template<class Type>
MyList<Type> :: MyList() {
   head = nullptr;
} 


// Delete all of the items in the list 
template<class Type>
MyList<Type> :: ~MyList() {
   MyNode<Type> *temp;  // temporary to delete each node
   MyNode<Type> *cur;   // for traversing
   cur = head;

   while (cur) {
      temp = cur;
      cur = cur->next;
      delete temp;
   }
   head = nullptr;
}
   

// Check if list is empty
template<class Type>
bool MyList<Type> :: IsEmpty() const {
   return (bool)(head == nullptr);
}



// Return data from first node
template<class Type>
Type MyList<Type> :: FrontOfList () const {
   assert (head); 
   return head->data;
}


// Traverse the list - display each item
template<class Type>
void MyList<Type> :: Traverse() const {
   MyNode<Type> *cur;	// pointer for traversing the list

   cout << "The list:" << endl;
   cur = head;
   while (cur) {       // note call to class method
      cout << cur->data << endl;
      cur = cur->next;
   }
}  


/***********************************************************
 NOTE: NOT STANDARD LIST ADT FUNCTIONS
***********************************************************/

// Calls recursive function; needed to send over head to start
template<class Type>
void MyList<Type> :: RecTraverse () const {
   cout << "The list:\n";
   RecTraverse2 (head);
}

// Display list backwards using recursion
template<class Type>
void MyList<Type> :: RecTraverse2 (MyNode<Type> *ptr) const {
   if (ptr) {
      RecTraverse2 (ptr->next);
      cout << ptr->data << endl;
   }
}
   
/**********************************************************/

// Insert new node into list in sorted order
// This will also update head pointers
template<class Type>
void MyList<Type> :: Insert (Type newdata) {
   MyNode<Type> *prev;     // pointer to previous node
   MyNode<Type> *newitem;  // pointer to newly created item
   MyNode<Type> *cur;      // pointer for traversing
   bool found;             // if position in list is found

   newitem = new MyNode<Type>;
   newitem->data = newdata;

   // find correct spot in list
   cur = head;
   prev = nullptr;
   found = false;

   while (cur && !found) {
      // if "data" is user defined, then must have overloaded < operator
      if (newitem->data < cur->data) 
         found = true;
      else {
         prev = cur;
         cur = cur->next;
      }
   }

   // now insert new item 
   if (prev) {
      newitem->next = cur;
      prev->next = newitem;
   }
   // insert at front of list
   else {
      newitem->next = head;
      head = newitem;
    cout << "inserting at front\n";
   }
}


// Delete an item from list; print message if item is not found
// Also updates head 
template<class Type>
void MyList<Type> :: Delete (Type delitem) {
   MyNode<Type> *prev;      // pointer to previous node
   MyNode<Type> *cur;       // pointer for traversing
   bool found = false;      // if item is found in list

   cur = head;
   prev = nullptr;
   
   while (cur && !found) {
      // another case where == may need overloaded operator
      if (delitem == cur->data)
         found = true;
      else {
         prev = cur;
         cur = cur->next;
      }
   }

   if (!found)
      cout << "Sorry - item " << delitem << " is not in the list\n";
   else if (prev == nullptr){
      head = cur->next;
      delete cur;
   }
   else {
      prev->next = cur->next;
      delete cur;
   }
}

// tell compiler the possible types that may ultimately be used
template class MyList<int>;
template class MyList<float>;
template class MyList<char>;
